! Copyright (c) 2020 Jeffrey Armstrong <jeff@rainbow-100.com>
!
! Permission is hereby granted, free of charge, to any person obtaining a copy
! of this software and associated documentation files (the "Software"), to deal
! in the Software without restriction, including without limitation the rights
! to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
! copies of the Software, and to permit persons to whom the Software is
! furnished to do so, subject to the following conditions:
!
! The above copyright notice and this permission notice shall be included in
! all copies or substantial portions of the Software.
!
! The Software shall be used for Good, not Evil.
!
! THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
! IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
! FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
! AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
! LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
! OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
! SOFTWARE.

module layout
implicit none

    integer, parameter::line_type_text             = 1
    integer, parameter::line_type_preformatted     = 2
    integer, parameter::line_type_link             = 3
    integer, parameter::line_type_indicator        = 99

    type::line
        character(len=:), allocatable::text
        integer::line_type
        
        integer, dimension(:), pointer::breaks
        
        type(line), pointer::next
    end type line

contains

    subroutine layout_lines(first_line, rendering_engine)
    use render
    implicit none
    
        type(line), pointer::first_line
        class(renderer)::rendering_engine
        
        type(line), pointer::walker
        logical::laying_out
        
        walker => first_line
        laying_out = .true.
        
        call rendering_engine%prepare_for_layout()
        
        do while(laying_out)
            if(.not. associated(walker%breaks)) then
                select case (walker%line_type)
                    
                    case (line_type_text)
                        walker%breaks => calculate_wrapping(rendering_engine, walker%text)
                    
                    case (line_type_preformatted)
                        walker%breaks => calculate_stop(rendering_engine, walker%text)
                        
                end select
            end if

            select case (walker%line_type)

                case (line_type_text)
                    call render_proportional(rendering_engine, walker%text, walker%breaks)

                case (line_type_preformatted)
                    call render_preformatted(rendering_engine, walker%text, walker%breaks(1))

                case (line_type_link)
                    call render_link(rendering_engine, walker%text)

            end select

            laying_out = associated(walker%next)
            if(laying_out) then
                walker => walker%next
            end if

        end do
        
        call rendering_engine%layout_complete()

    end subroutine layout_lines

    subroutine clear_line_breaks(first_line)
    implicit none

        type(line), intent(inout), pointer::first_line
        type(line), pointer::walker

        walker => first_line

        do while(associated(walker))
            if(associated(walker%breaks)) then
                deallocate(walker%breaks)
                walker%breaks => null()
            end if

            walker => walker%next

        end do

    end subroutine clear_line_breaks

    subroutine free_lines(first_line)
    implicit none

        type(line), intent(inout), pointer::first_line

        type(line), pointer::walker, past

        call clear_line_breaks(first_line)

        walker => first_line

        do while(associated(walker))
            if(allocated(walker%text)) then
                deallocate(walker%text)
            end if

            past => walker
            walker => walker%next

            deallocate(past)

        end do

        first_line => null()

    end subroutine free_lines

end module layout